import { PROJECT_NAME as name } from './constants.js'
import manifestMiddleware from './middlewares/manifest.js'
import proxyMiddleware from './middlewares/proxy.js'
import upgradeMiddleware from './middlewares/upgrade.js'
import detectDevServerMiddleware from './middlewares/detect-dev-server.js'

export default function (config) {
  return {

    enforce: 'pre',

    name,

    async configureServer ({ config: { server: serverConfig }, middlewares, httpServer }) {
      const proxies = Object.entries(config.proxy || {}).map(([condition, config]) => {
        // prepare proxy config
        const { hostname, protocol, port } = new URL(config.target)
        // TODO convert conditions starting with ^ into regex
        return [condition, {
          ...config,
          hostname,
          protocol,
          port
        }]
      })
      const webProxies = proxies.filter(([c, { ws }]) => ws !== true)
      const wsProxies = proxies.filter(([c, { ws }]) => ws === true)
      const frontendUrls = [].concat(config.frontendURIs).filter(Boolean).map(uri => new URL(uri))

      middlewares.use(detectDevServerMiddleware({
        httpServer,
        frontendUrls,
        serverConfig
      }))

      middlewares.use(manifestMiddleware({
        frontendUrls
      }))

      middlewares.use(proxyMiddleware({
        proxyURLs: webProxies,
        frontendUrls
      }))

      httpServer.on('upgrade', upgradeMiddleware({
        proxyURLs: wsProxies
      }))
    }
  }
}
