import parseurl from 'parseurl'
import * as proxy from '../proxy.js'
import * as http from '../http.js'

export default function ({ proxyURLs, frontendUrls }) {
  return async (req, res, next) => {
    if (req.headers['proxied-by-ox-proxy']) return next()

    const { path } = parseurl(req)
    for (const [condition, { hostname, protocol, port }] of proxyURLs) {
      if (path.startsWith(condition)) {
        console.log('proxy', path, 'due to condition', condition)
        try {
          return await proxy.web(req, res, { hostname, protocol, port })
        } catch (e) {
          console.log('Error: Could not proxy', path)
          res.statusCode = 404
          res.end()
          return
        }
      }
    }

    try {
      // TODO add caching, when a path was successful
      // TODO anticipate /@fs/ calls to the current directory
      console.log('find', path, 'in', frontendUrls.map(u => u.href))
      const { hostname, protocol, port } = await http.findResponsiveUrl({ urls: frontendUrls, path, headers: { accept: '*/*', 'proxied-by-ox-proxy': true } })
      console.log('found', path, 'in', protocol + '//' + hostname + ':' + port, 'and proxy it')
      return await proxy.web(req, res, { hostname, protocol, port, headers: { 'proxied-by-ox-proxy': true } })
    } catch (e) {
      console.log(`Could not find in frontends "${path}"`)
      res.statusCode = 404
      res.end()
    }
  }
}
