import { describe, it, expect, afterEach } from '@jest/globals'
import { createServer } from 'vite'
import path from 'path'
import vitePluginProxy from '../../src/index'
import vitePluginOxManifests from '@open-xchange/vite-plugin-ox-manifests'
import { getPort } from '../util'
import axios from 'axios'
import express from 'express'

const PORT = getPort()
const SERVER_PORT = getPort()
const __dirname = path.dirname(new URL(import.meta.url).pathname)

describe('frontend uris', function () {
  let server, expressServer

  afterEach(async function () {
    if (server) {
      await server.close()
      server = null
    }
    if (expressServer) {
      await expressServer.close()
      expressServer = null
    }
  })

  it('works', async function () {
    server = await createServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        frontendURIs: [`http://localhost:${SERVER_PORT}`]
      })]
    })
    await server.listen(PORT)

    // create dummy server
    const app = express()
    expressServer = app.listen(SERVER_PORT)
    app.get('/other.js', (req, res) => {
      res.send('console.log("Hello world")')
    })

    const { data: index } = await axios({ baseURL: `http://localhost:${PORT}/index.js` })
    expect(index).toEqual('window.test = \'foo\'\n')

    const { data: other } = await axios({ baseURL: `http://localhost:${PORT}/other.js` })
    expect(other).toEqual('console.log("Hello world")')
  })

  it('serves merged manifest.json', async function () {
    server = await createServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        frontendURIs: [`http://localhost:${SERVER_PORT}`]
      }), vitePluginOxManifests({
        // no config
      })]
    })
    await server.listen(PORT)

    // create dummy server
    const app = express()
    expressServer = app.listen(SERVER_PORT)
    app.get('/api/manifest.json', (req, res) => {
      res.json([{ namespace: 'other' }])
    })

    const { data } = await axios({ baseURL: `http://localhost:${PORT}/api/manifest.json` })
    expect(data).toEqual([{
      namespace: 'index',
      path: 'index'
    }, {
      namespace: 'other'
    }])
  })
})
