import { describe, it, expect, afterEach } from '@jest/globals'
import { createServer } from 'vite'
import path from 'path'
import vitePluginProxy from '../../src/index'
import { getPort } from '../util'
import axios from 'axios'
import express from 'express'

const PORT = getPort()
const SERVER_PORT = getPort()
const __dirname = path.dirname(new URL(import.meta.url).pathname)

describe('simple proxy', function () {
  let server, expressServer

  afterEach(async function () {
    if (server) {
      await server.close()
      server = null
    }
    if (expressServer) {
      await expressServer.close()
      expressServer = null
    }
  })

  it('works', async function () {
    server = await createServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        proxy: {
          '/api': {
            target: `http://localhost:${SERVER_PORT}`
          }
        }
      })]
    })
    await server.listen(PORT)

    // create dummy server
    const app = express()
    expressServer = app.listen(SERVER_PORT)
    app.get('/api/test', (req, res) => {
      res.json({ hello: 'world' })
    })

    const { data } = await axios({ baseURL: `http://localhost:${PORT}/api/test` })
    expect(data).toEqual({ hello: 'world' })
  })
})
